﻿using System;
using Newtonsoft.Json;
using System.Collections.Generic;
using System.Text.RegularExpressions;
using Oxide.Core.Libraries.Covalence;

namespace Oxide.Plugins {
    [Info("ConnectMessages", "Mroczny", "0.1.0")]
    [Description("Custom connect and disconnect messages.")]
    class ConnectMessages : CovalencePlugin {
        private class Response {
            [JsonProperty("country")]
            public string Country { get; set; }
            [JsonProperty("countrycode")]
            public string CountryCode { get; set; }
        }

        private void OnServerInitialized() {
            permission.RegisterPermission("connectmessages.admin", this);
#if HURTWORLD
			GameManager.Instance.ServerConfig.ChatConnectionMessagesEnabled = false;
#endif
        }

        protected override void LoadDefaultConfig() {
            LogWarning("Creating a new configuration file");
            Config["Show Admin Join Message"] = true;
            Config["Show Admin Leave Message"] = true;
            Config["Show Join Message"] = true;
            Config["Show Leave Message"] = true;
            Config["Use Country Message"] = true;
            Config["Use Country Code"] = false;
        }

        protected override void LoadDefaultMessages() {
            lang.RegisterMessages(new Dictionary<string, string> {
                ["Join Country Message"] = "[{1}] {0} joined the game.",
                ["Join Message"] = "{0} joined the game.",
                ["Leave Message"] = "{0} left the game."
            }, this);
            lang.RegisterMessages(new Dictionary<string, string> {
                ["Join Country Message"] = "[{1}] {0} dołączył do gry.",
                ["Join Message"] = "{0} dołączył do gry.",
                ["Leave Message"] = "{0} opuścił nas."
            }, this, "pl");
        }

        void OnUserConnected(IPlayer player) {
            if (!Convert.ToBoolean(Config["Show Join Message"]) || (!Convert.ToBoolean(Config["Show Admin Join Message"]) && player.IsAdmin || permission.UserHasPermission("connectmessages.admin", player.Id))) {
                return;
            }

            if (Convert.ToBoolean(Config["Use Country Message"])) {
                string apiUrl = "http://ip-api.com/json/";
                webrequest.Enqueue(apiUrl + player.Address, null, (code, response) => {
                    if (code != 200 || response == null) {
                        Puts($"WebRequest to {apiUrl} failed, sending connect message without the country.");
                        Broadcast("Join Message", player.Name);
                        return;
                    }

                    if (Convert.ToBoolean(Config["Use Country Code"])) {
                        string countrycode = JsonConvert.DeserializeObject<Response>(response).CountryCode;
                        Broadcast("Join Country Message", player.Name, countrycode);
                    } else {
                        string country = JsonConvert.DeserializeObject<Response>(response).Country;
                        Broadcast("Join Country Message", player.Name, country);
                    }
                }, this);
            }

            if (!Convert.ToBoolean(Config["Use Country Message"]) && Convert.ToBoolean(Config["Show Join Message"])) {
                Broadcast("Join Message", player.Name);
            }
        }

        void OnUserDisconnected(IPlayer player) {
            if (!Convert.ToBoolean(Config["Show Leave Message"]) || (!Convert.ToBoolean(Config["Show Admin Leave Message"]) && player.IsAdmin || permission.UserHasPermission("connectmessages.admin", player.Id))) {
                return;
            }
            if (Convert.ToBoolean(Config["Show Leave Message"])) {
                Broadcast("Leave Message", player.Name);
            }
        }

        private string StripRichText(string text) {
            var stringReplacements = new string[]
            {
                "<b>", "</b>",
                "<i>", "</i>",
                "</size>",
                "</color>"
            };

            var regexReplacements = new Regex[]
            {
                new Regex(@"<color=.+?>"),
                new Regex(@"<size=.+?>"),
            };

            foreach (var replacement in stringReplacements) {
                text = text.Replace(replacement, string.Empty);
            }

            foreach (var replacement in regexReplacements) {
                text = replacement.Replace(text, string.Empty);
            }

            return Formatter.ToPlaintext(text);
        }

        private void Broadcast(string msg, params object[] args) => server.Broadcast(StripRichText(string.Format(lang.GetMessage(msg, this), args)));
    }
}